Package: plotGrouper
Title: Shiny app GUI wrapper for ggplot with built-in statistical
        analysis
Version: 1.31.0
Authors@R: person("John D.", "Gagnon", 
  email = "john.gagnon.42@gmail.com", 
  role = c("aut", "cre"))
Description: A shiny app-based GUI wrapper for ggplot with built-in statistical 
  analysis. Import data from file and use dropdown menus and checkboxes to 
  specify the plotting variables, graph type, and look of your plots. 
  Once created, plots can be saved independently or stored in a report that 
  can be saved as a pdf. If new data are added to the file, the report can be 
  refreshed to include new data. Statistical tests can be selected and added to 
  the graphs. Analysis of flow cytometry data is especially integrated with 
  plotGrouper. Count data can be transformed to return the absolute
  number of cells in a sample (this feature requires inclusion of the number of 
  beads per sample and information about any dilution performed). 
Depends: R (>= 3.5)
Imports: ggplot2 (>= 3.0.0), dplyr (>= 0.7.6), tidyr (>= 0.2.0), tibble
        (>= 1.4.2), stringr (>= 1.3.1), readr (>= 1.1.1), readxl (>=
        1.1.0), scales (>= 1.0.0), stats, grid, gridExtra (>= 2.3), egg
        (>= 0.4.0), gtable (>= 0.2.0), ggpubr (>= 0.1.8), shiny (>=
        1.1.0), shinythemes (>= 1.1.1), colourpicker (>= 1.0), magrittr
        (>= 1.5), Hmisc (>= 4.1.1), rlang (>= 0.2.2)
Suggests: knitr, htmltools, BiocStyle, rmarkdown, testthat
VignetteBuilder: knitr
biocViews: ImmunoOncology, FlowCytometry, GraphAndNetwork,
        StatisticalMethod, DataImport, GUI, MultipleComparison
URL: https://jdgagnon.github.io/plotGrouper/
BugReports: https://github.com/jdgagnon/plotGrouper/issues
License: GPL-3
Encoding: UTF-8
RoxygenNote: 6.1.0
git_url: https://git.bioconductor.org/packages/plotGrouper
git_branch: devel
git_last_commit: 508fb9b
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-29 00:21:32 UTC; biocbuild
Author: John D. Gagnon [aut, cre]
Maintainer: John D. Gagnon <john.gagnon.42@gmail.com>
