% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotGrouper.R
\name{plotGrouper}
\alias{plotGrouper}
\title{A function to run the plotGrouper shiny app}
\usage{
plotGrouper(...)
}
\arguments{
\item{...}{Any argument that you can pass to shiny::runApp}
}
\value{
Runs the plotGrouper shiny app.
}
\description{
This function runs the plotGrouper app
}
\examples{
# plotGrouper()
}
